/******************************************************************************
 *
 * 	Barcode Scanner Common Defines.
 *
 *	Copyright (c) 2006	Janam Partners LLC. All rights reserved.
 *
 * 	File: 			JanamExtendedDef.h
 *
 * 	Release: 		Palm OS 5.x
 *
 * 	Description: 	Extended Defines for using the BABBO devices
 *
 * 	History:	
 *	Version 1.0		PT, JL			Oct, 2006
 *
 *****************************************************************************/
 
#ifndef __JANAMEXTENDEDDEF_H__
#define __JANAMEXTENDEDDEF_H__

//Type Defines 
#define LONG long
#define WORD Int16
#define DWORD Int32

//The Max BarCode Length
#define MAXBARCODELENGTH	3750
//-----------------------------------------------------------------------------

//Barcode data types returned

//Added to the current library
#define BCTYPE_CODE32                               0x20    
#define BCTYPE_PDF417								0x11 

//Extended support by BABBO 
//Defined at a range of 0x31-0x3F, 0x60-0x6D
#define BCTYPE_AZTEC  								0x31       
#define BCTYPE_CODE11								0x32
#define BCTYPE_CODE49        						0x33
#define BCTYPE_COMPOSITE     						0x34
#define BCTYPE_DATAMATRIX   						0x35
#define BCTYPE_MAXICODE     						0x36
#define BCTYPE_MICROPDF      						0x37
#define BCTYPE_POSTNET     							0x38
#define BCTYPE_OCR         							0x39
#define BCTYPE_QR           						0x3A		
#define BCTYPE_RSS         							0x3B
#define BCTYPE_ISBT         						0x3C
#define BCTYPE_BPO          						0x3D
#define BCTYPE_CANPOST      						0x3E
#define BCTYPE_AUSPOST       						0x3F

#define BCTYPE_IATA25       						0x60
#define BCTYPE_CODABLOCK    						0x61
#define BCTYPE_JAPOST       						0x62
#define BCTYPE_PLANET       						0x63
#define BCTYPE_DUTCHPOST     						0x64
#define BCTYPE_MATRIX25	    						0x65
#define BCTYPE_CHINAPOST							0x66
#define BCTYPE_KOREAPOST							0x67
#define BCTYPE_TELEPEN								0x68
#define BCTYPE_CODE16K								0x69
#define BCTYPE_POSICODE								0x6A
#define BCTYPE_COUPONCODE							0x6B
#define BCTYPE_USPS4CB       						0x6C
#define BCTYPE_IDTAG								0x6D

//-----------------------------------------------------------------------------
// Decoder configuration definitions for each symbology
#define SYMBOLOGY_ENABLE                0x00000001  // Enable Symbology bit
#define SYMBOLOGY_CHECK_ENABLE          0x00000002  // Enable usage of check character
#define SYMBOLOGY_CHECK_TRANSMIT        0x00000004  // Send check character
#define SYMBOLOGY_START_STOP_XMIT       0x00000008  // Include the start and stop characters in the decoded result string
#define SYMBOLOGY_ENABLE_APPEND_MODE    0x00000010  // Code39 append mode
#define SYMBOLOGY_ENABLE_FULLASCII      0x00000020  // Enable Code39 Full ASCII
#define SYMBOLOGY_ENABLE_BASE32         0x00000040  // Code32 (PARAF) enable
#define SYMBOLOGY_NUM_SYS_TRANSMIT      0x00000040  // UPC-A/UPC-e Send Num Sys
#define SYMBOLOGY_2_DIGIT_ADDENDA       0x00000080  // Enable 2 digit Addenda (UPC & EAN)
#define SYMBOLOGY_5_DIGIT_ADDENDA       0x00000100  // Enable 5 digit Addenda (UPC & EAN)
#define SYMBOLOGY_ADDENDA_REQUIRED      0x00000200  // Only allow codes with addenda (UPC & EAN)
#define SYMBOLOGY_ADDENDA_SEPARATOR     0x00000400  // Include Addenda separator space in returned string.
#define SYMBOLOGY_EXPANDED_UPCE         0x00000800  // Extended UPC-E
#define SYMBOLOGY_ISBT_CONCATENATE		0x00000040  // C128 ISBT concatenation
#define SYMBOLOGY_COMPOSITE_UPC         0x00002000  // Enable UPC composite codes
#define SYMBOLOGY_AZTEC_RUNE            0x00004000  // Enable Aztec Run
#define SYMBOLOGY_AUSTRALIAN_BAR_WIDTH  0x00010000  // Include australian postal bar data in string
// Telepen and PosiCode
#define SYMBOLOGY_TELEPEN_OLD_STYLE     0x04000000  // Telepen Old Style mode.
#define SYMBOLOGY_POSICODE_LIMITED_1    0x08000000  // PosiCode Limited of 1
#define SYMBOLOGY_POSICODE_LIMITED_2    0x10000000  // PosiCode Limited of 2
#define SYMBOLOGY_CODABAR_CONCATENATE   0x20000000  // Codabar concatenate.
#define SYMBOLOGY_CODABAR_CONCAT_REQ    0x40000000  // Codabar concatenate required.
#define SYMBOLOGY_EAN13_ISBN_XLATE      0x00080000  // EAN 13-Bookland- ISBN


//-----------------------------------------------------------------------------

// Symbology structure set masks
#define SYM_MASK_FLAGS                  0x00000001  // Flags are valid
#define SYM_MASK_MIN_LEN                0x00000002  // Min Length valid
#define SYM_MASK_MAX_LEN                0x00000004  // Max Length valid
#define SYM_MASK_ALL                    0x00000007	// All fields valid

//-----------------------------------------------------------------------------

// Define aliases for each symbology structure
#define AZTEC_T         SymFlagsRange_t
#define CODABAR_T       SymFlagsRange_t
#define CODE11_T        SymFlagsRange_t
#define CODE128_T       SymFlagsRange_t
#define CODE39_T        SymFlagsRange_t
#define CODE49_T        SymFlagsRange_t
#define CODE93_T        SymFlagsRange_t
#define COMPOSITE_T     SymFlagsRange_t
#define DATAMATRIX_T    SymFlagsRange_t
#define EAN8_T          SymFlagsOnly_t
#define EAN13_T         SymFlagsOnly_t
#define INT25_T         SymFlagsRange_t
#define MAXICODE_T      SymFlagsRange_t
#define MICROPDF_T      SymFlagsRange_t
#define PDF417_T        SymFlagsRange_t
#define POSTNET_T       SymFlagsOnly_t
#define QR_T            SymFlagsRange_t
#define RSS_T           SymFlagsOnly_t
#define RSSLIM_T		SymFlagsOnly_t
#define RSSEXP_T		SymFlagsRange_t
#define UPCA_T          SymFlagsOnly_t
#define UPCE_T          SymFlagsOnly_t
#define UPCE1_T         SymFlagsOnly_t
#define BPO_T           SymFlagsOnly_t
#define CANPOST_T       SymFlagsOnly_t
#define AUSPOST_T       SymFlagsOnly_t
#define IATA25_T        SymFlagsRange_t
#define CODABLOCK_T     SymFlagsRange_t
#define JAPOST_T        SymFlagsOnly_t
#define PLANET_T        SymFlagsOnly_t
#define DUTCHPOST_T     SymFlagsOnly_t
#define MSI_T           SymFlagsRange_t
#define TLCODE39_T      SymFlagsOnly_t
#define MATRIX25_T      SymFlagsRange_t
#define KORPOST_T       SymFlagsRange_t
#define TRIOPTIC_T      SymFlagsOnly_t
#define CODE25_T        SymFlagsRange_t
#define PLESSEY_T       SymFlagsRange_t
#define CHINAPOST_T     SymFlagsRange_t
#define TELEPEN_T       SymFlagsRange_t
#define CODE16K_T       SymFlagsRange_t
#define POSICODE_T      SymFlagsRange_t
#define COUPONCODE_T    SymFlagsOnly_t
#define USPS4STATE_T	SymFlagsOnly_t
#define IDTAG_T			SymFlagsOnly_t

//define for exposure settings
#define DEC_ES_EXPOSURE_METHOD 0
#define DEC_ES_TARGET_VALUE 1
#define DEC_ES_TARGET_PERCENTILE 2
#define DEC_ES_TARGET_ACCEPT_GAP 3
#define DEC_ES_MAX_EXP 4
#define DEC_ES_MAX_GAIN 5
#define DEC_ES_STARTING_EXP 6
#define DEC_ES_STARTING_GAIN 7
#define DEC_ES_FRAME_RATE 8
#define DEC_ES_CONFORM_IMAGE 9
#define DEC_ES_CONFORM_TRIES 10
#define DEC_ES_SPECULAR_EXCLUSION 11
#define DEC_ES_SPECULAR_SAT 12
#define DEC_ES_SPECULAR_LIMIT 13
#define DEC_ES_FIXED_EXP 14
#define DEC_ES_FIXED_GAIN 15
#define DEC_ES_FIXED_FRAME_RATE 16
#define DEC_ES_CENTER_SAMPLE_WINDOW_TOP 18
#define DEC_ES_CENTER_SAMPLE_WINDOW_BOT 19
#define DEC_ES_CENTER_SAMPLE_WINDOW_LFT 20
#define DEC_ES_CENTER_SAMPLE_WINDOW_RGT 21
#define DEC_ES_CENTER_SAMPLE_WINDOW_H_PER 22
#define DEC_ES_CENTER_SAMPLE_WINDOW_V_PER 23
#define DEC_ES_MIN_EXP 24

#define DEC_MIN_ES_EXPOSURE_METHOD  0
#define DEC_MAX_ES_EXPOSURE_METHOD  2
#define DEC_MIN_ES_TARGET_VALUE 1
#define DEC_MAX_ES_TARGET_VALUE 255
#define DEC_MIN_ES_TARGET_PERCENTILE 1
#define DEC_MAX_ES_TARGET_PERCENTILE 99
#define DEC_MIN_ES_TARGET_ACCEPT_GAP 1
#define DEC_MAX_ES_TARGET_ACCEPT_GAP 50
#define DEC_MIN_ES_MAX_EXP 1
#define DEC_MAX_ES_MAX_EXP 7874
#define DEC_MIN_ES_MAX_GAIN 1
#define DEC_MAX_ES_MAX_GAIN 4
#define DEC_MIN_ES_FRAME_RATE 1
#define DEC_MAX_ES_FRAME_RATE 30
#define DEC_MIN_ES_CONFORM_IMAGE 0
#define DEC_MAX_ES_CONFORM_IMAGE 1
#define DEC_MIN_ES_CONFORM_TRIES 1
#define DEC_MAX_ES_CONFORM_TRIES 8
#define DEC_MIN_ES_SPECULAR_EXCLUSION 0
#define DEC_MAX_ES_SPECULAR_EXCLUSION 4
#define DEC_MIN_ES_SPECULAR_SAT 200
#define DEC_MAX_ES_SPECULAR_SAT 255
#define DEC_MIN_ES_SPECULAR_LIMIT 1
#define DEC_MAX_ES_SPECULAR_LIMIT 5
#define DEC_MIN_ES_FIXED_EXP 1
#define DEC_MAX_ES_FIXED_EXP 7874
#define DEC_MIN_ES_FIXED_GAIN 1
#define DEC_MAX_ES_FIXED_GAIN 4
#define DEC_MIN_ES_FIXED_FRAME_RATE 1
#define DEC_MAX_ES_FIXED_FRAME_RATE 30
#define DEC_MIN_ES_CENTER_SAMPLE_WINDOW_TOP 0
#define DEC_MAX_ES_CENTER_SAMPLE_WINDOW_TOP 479
#define DEC_MIN_ES_CENTER_SAMPLE_WINDOW_BOT 0
#define DEC_MAX_ES_CENTER_SAMPLE_WINDOW_BOT 479
#define DEC_MIN_ES_CENTER_SAMPLE_WINDOW_LFT 0
#define DEC_MAX_ES_CENTER_SAMPLE_WINDOW_LFT 751
#define DEC_MIN_ES_CENTER_SAMPLE_WINDOW_RGT 0
#define DEC_MAX_ES_CENTER_SAMPLE_WINDOW_RGT 751
#define DEC_MIN_ES_CENTER_SAMPLE_WINDOW_H_PER 4
#define DEC_MAX_ES_CENTER_SAMPLE_WINDOW_H_PER 748
#define DEC_MIN_ES_CENTER_SAMPLE_WINDOW_V_PER 1
#define DEC_MAX_ES_CENTER_SAMPLE_WINDOW_V_PER 751
#define DEC_MIN_ES_MIN_EXP 1
#define DEC_MAX_ES_MIN_EXP 7874

#define DEC_AE_METHOD_UNIFORM 0
#define DEC_AE_METHOD_CENTER_ONLY 1
#define DEC_AE_METHOD_CENTER_WEIGHTED 2

//-----------------------------------------------------------------------------

#endif
