/******************************************************************************
 *
 * 	Barcode Scanner Extended Struct Defines.
 *
 *	Copyright (c) 2006	Janam Partners LLC. All rights reserved.
 *
 * 	File: 			JanamExtendedStruct.h
 *
 * 	Release: 		Palm OS 5.x
 *
 * 	Description: 	Extended Struct and Enum for using BABBO devices
 *
 * 	History:	
 *	Version 1.0		PT, JL			Oct, 2006
 *
 *****************************************************************************/
 

#ifndef __JANAMEXTENDEDSTRUCT_H__
#define __JANAMEXTENDEDSTRUCT_H__

 
// Image formats Enum
typedef enum
{
    IF_RAW_BINARY = 0,
    IF_RAW_GRAY
} ImageFormat_t;

// OCR Mode Enum
typedef enum
{
    OCR_DISABLED = 0,
    OCR_A,
    OCR_B,
    OCR_MONEY,
    OCR_MICR
}OCRMode_t;

// Query Type Enum
typedef enum
{
    QT_DEFAULT = 0,
    QT_CURRENT
} QueryType_t;

//-----------------------------------------------------------------------------
// Lights mode used during WaitForDecode function calls.
//-----------------------------------------------------------------------------
typedef enum
{
    LM_ILLUM_AIMER_OFF=0,	// Neither aimers or illumination
    LM_AIMER_ONLY,		    // Aimer only
    LM_ILLUM_ONLY,			// Illum only
    LM_ILLUM_AIMER			// Alternating Aimer & illumination(default)
} LightsMode_t;
 
 //-----------------------------------------------------------------------------


// Structure for symbologies with no specified min or max length.
typedef struct _tagSymFlagsOnly
{
    Int32   dwStructSize;                           // Set to sizeof( SymFlagsOnly_t );
    Int32   dwMask;
    Int32   dwFlags;

} SymFlagsOnly_t, *PSymFlagsOnly_t;


// Structure for symbologies with min & max length.
typedef struct _tagSymFlagsRange
{
    Int32   dwStructSize;                           // Set to sizeof( SymFlagsRange_t );
    Int32   dwMask;
    Int32   dwFlags;
    Int32   dwMinLen;
    Int32   dwMaxLen;

} SymFlagsRange_t, *PSymFlagsRange_t;

// Structure of structures, one for each symbology.
typedef struct _tagSymCfg
{
    Int32           dwStructSize;   // Set to sizeof( SymCfg_t );
    // Linear Codes                 // Flags supported for this code
    //---------------------------------------------------------------
    CODABAR_T       codabar;        // Enable,Check,CheckSend,StartStop,Concatenate
    CODE11_T        code11;         // Enable,Check,CheckSend
    CODE128_T       code128;        // Enable
    CODE39_T        code39;         // Enable,Check,CheckSend,StartStop,Append,FullAscii
    CODE49_T        code49;         // Enable
    CODE93_T        code93;         // Enable
    COMPOSITE_T     composite;      // Enable,CompositeUPC
    DATAMATRIX_T    datamatrix;     // Enable
    EAN8_T          ean8;           // Enable,Check,Addenda2,Addenda5,AddendaReq,AddendaSep
    EAN13_T         ean13;          // Enable,Check,Addenda2,Addenda5,AddendaReq,AddendaSep,ISBN Translate
    IATA25_T        iata25;         // Enable
    INT25_T         int2of5;        // Enable,Check,CheckSend
    MSI_T           msi;            // Enable,Check
    UPCA_T          upcA;           // Enable,check,NumSysTrans,Addenda2,Addenda5,AddendaReq,AddendaSep
    UPCE_T          upcE;           // Enable,check,NumSysTrans,Addenda2,Addenda5,AddendaReq,AddendaSep,ExpandedE,
    UPCE1_T         upcE1;           // Enable
    // Postal Codes
    AUSPOST_T       australiaPost;  // Enable,AustralianBar
    BPO_T           britishPost;    // Enable
    CANPOST_T       canadaPost;     // Enable
    DUTCHPOST_T     dutchPost;      // Enable
    JAPOST_T        japanPost;      // Enable
    PLANET_T        usPlanet;       // Enable,Check
    POSTNET_T       usPostnet;      // Enable,Check
    // 2D Codes
    AZTEC_T         aztec;          // Enable,AztecRune
    CODABLOCK_T     codablock;      // Enable
    MAXICODE_T      maxicode;       // Enable
    MICROPDF_T      microPDF;       // Enable
    PDF417_T        pdf417;         // Enable
    QR_T            qr;             // Enable
    RSS_T           rss;            // Enable
    TLCODE39_T      tlCode39;       // Enable
    // New codes
    MATRIX25_T      matrix25;       // Enable,
    KORPOST_T       koreaPost;      // Enable
    TRIOPTIC_T      triopticCode;   // Enable
    CODE25_T        code2of5;       // Enable
    PLESSEY_T       plesseyCode;    // Enable
    CHINAPOST_T     chinaPost;      // Enable
    TELEPEN_T       telepen;        // Enable,OldStyle?
    CODE16K_T       code16k;        // Enable
    POSICODE_T      posiCode;       // Enable,Limited 1 and 2
    COUPONCODE_T    couponCode;     // Enable
    USPS4STATE_T	usps4Post;		// Enable
    IDTAG_T		    idtagPost;      // Enable
	RSSLIM_T		rssLim;			// Enable
	RSSEXP_T		rssExp;			// Enable
} SymCfg_t, *PSymCfg_t;


// Intell Img Structure
typedef struct
{
    int AspectRatio;            // ratioof barcode height to narrow elem width
    int OffsetX;                // offset in X direction, relative to barcode center
    int OffsetY;                // offset in Y direction
    unsigned int width;         // width of image in IntellBarcodeUnits
    unsigned int height;        // height of image
    int resolution;             // # pixels/IntellBarcodeUnits
    ImageFormat_t format;
	unsigned int destNaturalWidthInPixels;	// (OUT) base to report native DPI
	unsigned int destNaturalHeightInPixels;
	unsigned int destWidthInPixels;	// (IN) alternative to resolution (if resolution = 0)
	unsigned int destHeightInPixels;
} IntellImgDesc_t;

// Image Properties Structure
typedef struct
{
	DWORD	dwSize;
	DWORD	dwEngineID;
    DWORD   dwImagerRows;
    DWORD   dwImagerCols;
    DWORD   dwBitsPerPixel;
	DWORD	dwRotation;
	DWORD	dwAimerXoffset;
	DWORD	dwAimerYoffset;
	DWORD	dwYDepth;
} ImagerProperties_t;

//Image Atributes Structure
typedef struct
{
    int     Size;               // Size of image
    int		Exposure;			//
	int		Gain;				//
    int		IllumValue;			//
	int		IllumMax;			//
    int		IllumClip;			//
} ImageAttributes_t;


//Decode RAW data Structure
typedef struct
{
    Int8*   pchMessage;
    Int8    chCodeID;
    Int8    chSymLetter;
    Int8    chSymModifier;
    Int16    nLength;
} DecodeMsgRaw_t;


//RECT structure
typedef struct _RECT {
	LONG left;
	LONG top;
	LONG right;
	LONG bottom;
} RECT, *PRECT, *LPRECT;


 typedef enum
    {
	    ES_EXPOSURE_METHOD,		/* How should we do this */
    	ES_TARGET_VALUE,		/* This is the target pixel count we shoot for */
	    ES_TARGET_PERCENTILE,	/* This goes along with Value, the target value should be at this percentile */
    	ES_TARGET_ACCEPT_GAP,	/* How close to the target value must we be */
	    ES_MAX_EXP,				/* This is the maximum exposure we're allowed to use */
    	ES_MAX_GAIN,			/* This is the maximum gain we're allowed to use */
	    ES_STARTING_EXP,		/* When scanning starts, this is the first exposure that we try (updated when scanning stops)*/
    	ES_STARTING_GAIN,		/* When scanning starts, this is the first gain that we try (updated when scanning stops)*/
	    ES_FRAME_RATE,			/* This is essentially the frame rate to use, I call it divide because that's what it does */
    	ES_CONFORM_IMAGE,		/* The image must conform to the auto-exposure requirements, if not, it's rejected. */
	    ES_CONFORM_TRIES,		/* The number of times we'll attempt to conform. */
    	ES_SPECULAR_EXCLUSION,	/* Do we exclude specular? to what degree? */
	    ES_SPECULAR_SAT,		/* These next two only come into play if somebody specifies nSpecularExclusion*/
    	ES_SPECULAR_LIMIT,		/* to be HHPSD_SPECULAR_EXCLUSION_SPECIAL. */
	    ES_FIXED_EXP,
    	ES_FIXED_GAIN,
	    ES_FIXED_FRAME_RATE,
		ES_ADJUST_EXP_WHEN_ILLUM_OFF,
		ES_CENTER_SAMPLE_WINDOW_TOP,
		ES_CENTER_SAMPLE_WINDOW_BOT,
		ES_CENTER_SAMPLE_WINDOW_LFT,
		ES_CENTER_SAMPLE_WINDOW_RGT,
		ES_CENTER_SAMPLE_WINDOW_H_PER,
		ES_CENTER_SAMPLE_WINDOW_V_PER,
		ES_MIN_EXP,
        ES_NUM_EXPOSURE_ITEMS
    } HHP_IMAGER_EXPOSURE_TAG;

typedef enum {
	HHPSD_IOCTL_START_SCANNING,
	HHPSD_IOCTL_STOP_SCANNING,
	HHPSD_IOCTL_GET_NEW_SCAN,
	HHPSD_IOCTL_UNREGISTER_SCAN_BUFFER,
	HHPSD_IOCTL_REGISTER_SCAN_BUFFER,
	HHPSD_IOCTL_GET_SCAN_HEIGHT,
	HHPSD_IOCTL_GET_SCAN_WIDTH,
	HHPSD_IOCTL_SET_ILLUMINATORS_STATE,
	HHPSD_IOCTL_SET_AIMER_STATE,
	HHPSD_IOCTL_SET_EXPOSURE_SETTINGS,
	HHPSD_IOCTL_GET_EXPOSURE_SETTINGS,
	HHPSD_IOCTL_RETURN_SCAN,
	HHPSD_IOCTL_SET_SCAN_LIGHTS_MODE,
	HHPSD_IOCTL_GET_ENGINE_TYPE,
	HHPSD_IOCTL_SET_EXPOSURE_MODE,
	HHPSD_IOCTL_SET_ILLUMINATOR_INTENSITY,
	HHPSD_IOCTL_SET_AIMER_INTENSITY,
	HHPSD_IOCTL_NEW_IMAGE_AVAILABLE,
	HHPSD_IOCTL_PREVENT_AIMER_IN_IMAGES,
	HHPSD_IOCTL_GET_IMAGE_ATTRIBUTES,
	HHPSD_IOCTL_RECONFIG_ENGINE,
    HHPSD_IOCTL_SINGLE_FRAME_CAPTURE,
	HHPSD_IOCTL_GET_REVISION_STRING,
	HHPSD_IOCTL_GET_IMAGER_PROPERTIES,
	HHPSD_IOCTL_POWER_OFF_IMAGER,
	HHPSD_IOCTL_SET_ENGINE_CONFIG,
	HHPSD_IOCTL_GET_ENGINE_CONFIG,
	HHPSD_IOCTL_WRITE_PSOC_REG,
	HHPSD_IOCTL_READ_PSOC_REG,
	HHPSD_IOCTL_WRITE_PSOC,
	HHPSD_IOCTL_READ_PSOC
//	HHPSD_IOCTL_WRITE_I2C_PSOC,
//	HHPSD_IOCTL_READ_I2C_PSOC
}HHPScanDriverIOCTL_t;

typedef enum {
	HHPSD_EXPOSURE_FIXED,
	HHPSD_AUTOEXPOSURE_USE_ONCHIP,
	HHPSD_AUTOEXPOSURE_USE_HHP
}HHPScanDriverAutoExposureMode_t;

typedef enum {
	HHPSD_ILLUM_AIMER_OFF,
	HHPSD_ILLUM_ONLY_ON,
	HHPSD_AIMER_ONLY_ON,
	HHPSD_ILLUM_AIMER_ON
} HHPScanDriverIlluminationStates_t;

typedef enum {
	HHPSD_AE_METHOD_UNIFORM,
	HHPSD_AE_METHOD_CENTER_ONLY,
	HHPSD_AE_METHOD_CENTER_WEIGHTED
} HHPScanDriverHHPExposureMethod;

typedef enum {
	HHPSD_SPECULAR_EXCLUSION_OFF,
	HHPSD_SPECULAR_EXCLUSION_MINIMAL,
	HHPSD_SPECULAR_EXCLUSION_MODERATE,
	HHPSD_SPECULAR_EXCLUSION_AGGRESSIVE,
	HHPSD_SPECULAR_EXCLUSION_SPECIAL
} HHPScanDriverHHPExposureSpecularExclusion;

typedef enum {
	HHPSD_ENGINE_IMAGER_OV7120,
	HHPSD_ENGINE_IMAGER_ICMEDIA,
	HHPSD_ENGINE_LINEAR_IMAGER_TOSHIBA,
	HHPSD_ENGINE_IMAGER_ALD_ICMEDIA,
	HHPSD_ENGINE_IMAGER_MI1300,
	HHPSD_ENGINE_IMAGER_STM_VC700,
	HHPSD_ENGINE_IMAGER_STM_VC602,
	HHPSD_ENGINE_IMAGER_MICRON_MT9V022
 } HHPScanDriverEngineType_t;

typedef enum {
	HHPSD_IMAGE_STAT_EXPOSURE,
	HHPSD_IMAGE_STAT_GAIN,
	HHPSD_IMAGE_STAT_ILLUM_VALUE,
	HHPSD_IMAGE_STAT_ILLUM_MAXIMUM,
	HHPSD_IMAGE_STAT_ILLUM_CLIP,
	HHPSD_IMAGE_STAT_HEIGHT,
	HHPSD_IMAGE_STAT_WIDTH
} HHPScanDriverImageAttributes_t;


typedef enum {
	IP_SIZE,
	IP_ENGINE_ID,
	IP_NUM_ROWS,
	IP_NUM_COLS,
	IP_BITS_PER_PIXEL,
	IP_ROTATION,
	IP_AIMER_X_OFFSET,
	IP_AIMER_Y_OFFSET,
	IP_YDEPTH
} HHPScanDriverImagerProperties_t;


typedef enum {
	HHPSD_ILLUMINATION_TYPE,
	HHPSD_LED_CONTROL_MODE,
	HHPSD_PIXEL_CLOCK_FREQUENCY,
	HHPSD_ENGINE_ID,
	HHPSD_ENGINE_FW_CKSUM,
	HHPSD_ENGINE_FW_VERSION,
	HHPSD_ENGINE_AIMER_X_CENTER,
	HHPSD_ENGINE_AIMER_Y_CENTER,
	HHSD_NUM_ENGINE_CFG_ITEMS
} HHPScanDriverEngineConfiguration_t;


#endif
