/******************************************************************************
 *
 * 	Barcode Scanner Demo Application
 *
 *	Copyright (c) 2006	Janam Partners LLC. All rights reserved.
 *
 * 	File: 			ScanMgr.h
 *
 * 	Release: 		Palm OS 5.x
 *
 * 	Description: 	Reserved for backward compatible
 *					Utility functions header
 *
 * 	History:	
 *	Version 1.0		PT, JL			Oct, 2006
 *
 *****************************************************************************/
 
//#pragma once
#ifndef __SCANMGR_H__
#define __SCANMGR_H__

#ifdef __cplusplus
   extern "C" {
#endif

#include "ScanMgrStruct.h"
#include "ScanMgrDef.h"
#include "ScanWedge.h"
#include "JanamExtended.h"


//--------------------------------------------------------------------------------------------------
//Check if it is a Janam device
Int16 ScnIsJanamUnit();

//--------------------------------------------------------------------------------------------------

//Connect or disconnect the barcode scanner library
Int16 ScnOpenDecoder();
Int16 ScnCloseDecoder();

//Send or retrieve parameters
Int16 ScnCmdSendParams( BeepType beep );
Int16 ScnCmdGetAllParams( UInt8* pbParams, UInt16 max_length );

//Inquire current scanner/decoder/port driver versions
Int16 ScnGetScanManagerVersion( Char *  pszVer, UInt16 max_length );
Int16 ScnGetScanPortDriverVersion( Char *  pszVer, UInt16 max_length );
Int16 ScnGetDecoderVersion( Char *  pszVer, UInt16 max_length);

//Get the decode and extended data
Int16 ScnGetDecodedData( MESSAGE *ptr);
Int16 ScnGetExtendedDecodedData( Int16 length, Int16 *type, UInt8 *extendedData);

//Enable or disable a barcode symbology
Int16 ScnSetBarcodeEnabled( BarType barcodeType, Boolean bEnable );
Int16 ScnGetBarcodeEnabled( BarType barcodeType );

//Set and get the barcode length of a barcode symbology
Int16 ScnSetBarcodeLengths( BarType barcodeType, UInt16 lengthType, UInt16 length1, UInt16 length2 );
Int16 ScnGetBarcodeLengths( BarType barcodeType, UInt16* pLengthType, UInt16* pLength1, UInt16* pLength2 );

//--------------------------------------------------------------------------------------------------

//UpcPreamble 
Int16 ScnSetUpcPreamble( BarType barcodeType, Int16 preamble);
Int16 ScnGetUpcPreamble( BarType barcodeType);

//Prefix Suffix
Int16 ScnSetPrefixSuffixValues( Int8 prefix, Int8 suffix_1, Int8 suffix_2 );
Int16 ScnGetPrefixSuffixValues( Char *  pPrefix, Char *  pSuffix_1, Char *  pSuffix_2 );

//Code32
Int16 ScnSetCode32Prefix( Boolean bEnable );
Int16 ScnGetCode32Prefix(void);

//Conversion
Int16 ScnSetConvert( ConvertType conversion, Boolean bEnable);
Int16 ScnGetConvert( ConvertType conversion);

//Check Digit
Int16 ScnSetTransmitCheckDigit( BarType barType, UInt16 check_digit );
Int16 ScnGetTransmitCheckDigit( BarType barType );

//Code 39 Check Digit Verification
Int16 ScnSetCode39CheckDigitVerification(UInt16 check_digit);
Int16 ScnGetCode39CheckDigitVerification(void);

//I2of5 Check Digit Verification
Int16 ScnSetI2of5CheckDigitVerification( UInt16 check_digit );
Int16 ScnGetI2of5CheckDigitVerification(void);

//Msi Plessy Check Digit
Int16 ScnSetMsiPlesseyCheckDigits( UInt16 check_digits );
Int16 ScnGetMsiPlesseyCheckDigits();

//Msi Plessy Check Algorithm
Int16 ScnSetMsiPlesseyCheckDigitAlgorithm( UInt16 algorithm );
Int16 ScnGetMsiPlesseyCheckDigitAlgorithm();

//Upc Ean Supplementals
Int16 ScnSetDecodeUpcEanSupplementals(  UInt16 supplementals );
Int16 ScnGetDecodeUpcEanSupplementals();

//Upc Ean Reducdancy
Int16 ScnSetDecodeUpcEanRedundancy(  UInt16 supplemental_redundancy );
Int16 ScnGetDecodeUpcEanRedundancy();

//Code 39 Full ASCII
Int16 ScnSetCode39FullAscii( Boolean bEnable );
Int16 ScnGetCode39FullAscii(void);

//CLSI Editing
Int16 ScnSetClsiEditing( Boolean bEnable );
Int16 ScnGetClsiEditing();

//Notis Editing
Int16 ScnSetNotisEditing( Boolean bEnable );
Int16 ScnGetNotisEditing();

//UPC EAN Security Level
Int16 ScnSetUpcEanSecurityLevel( UInt16 security_level );
Int16 ScnGetUpcEanSecurityLevel();

//EAN Zero Extend
Int16 ScnSetEanZeroExtend( Boolean bEnable );
Int16 ScnGetEanZeroExtend();

//Time out
Int16 ScnSetHostSerialResponseTimeOut( UInt16 time_out );
Int16 ScnGetHostSerialResponseTimeOut();

//--------------------------------------------------------------------------------------------------

//Reset to factory default
Int16 ScnCmdParamDefaults();

//Enable or disable Scanning function
Int16 ScnCmdScanEnable();
Int16 ScnCmdScanDisable();

//Start or stop decode function
Int16 ScnCmdStartDecode();
Int16 ScnCmdStopDecode();

//LED On Off
Int16 ScnCmdLedOn();
Int16 ScnCmdLedOff();

//Aimer On Off
Int16 ScnCmdAimOn();
Int16 ScnCmdAimOff();

//Get current status
Int16 ScnGetAimMode();
Int16 ScnGetScanEnabled();
Int16 ScnGetLedState();

//--------------------------------------------------------------------------------------------------

//Set Laser
Int16 ScnSetLaserOnTime( UInt16 laser_on_time);
Int16 ScnGetLaserOnTime();

//Set decode led on time
Int16 ScnSetDecodeLedOnTime( UInt16 led_on_time);
Int16 ScnGetDecodeLedOnTime();

//Set Angle
Int16 ScnSetAngle( UInt16 scan_angle );
Int16 ScnGetAngle();

//Set aim duration
Int16 ScnSetAimDuration( UInt16 aim_duration);
Int16 ScnGetAimDuration();

//Set triggering modes
Int16 ScnSetTriggeringModes( UInt16 triggering_mode);
Int16 ScnGetTriggeringModes();

//Set time out
Int16 ScnSetTimeOutBetweenSameSymbol( UInt16 time_out );
Int16 ScnGetTimeOutBetweenSameSymbol();

//Set Linear Security Level
Int16 ScnSetLinearCodeTypeSecurityLevel( UInt16 security_level );
Int16 ScnGetLinearCodeTypeSecurityLevel();

//Set Bidirectional Redundancy
Int16 ScnSetBidirectionalRedundancy( UInt16 redundancy );
Int16 ScnGetBidirectionalRedundancy();

//Set Transmit Code Id Character
Int16 ScnSetTransmitCodeIdCharacter( UInt16 code_id );
Int16 ScnGetTransmitCodeIdCharacter();

//Set Scan Data Transmission
Int16 ScnSetScanDataTransmissionFormat( UInt16 transmission_format );
Int16 ScnGetScanDataTransmissionFormat();

//--------------------------------------------------------------------------------------------------

//Beep Sound
Int16 ScnCmdBeep( BeepType beep );

//Set beep sound for decoding
Int16 ScnSetBeepAfterGoodDecode( Boolean bEnableBeep );
Int16 ScnGetBeepAfterGoodDecode();

//Set beep sound frequency
Int16 ScnSetBeepFrequency( FrequencyType type, Int16 beep_freq );
Int16 ScnGetBeepFrequency( FrequencyType type );

//Set beep duration
Int16 ScnSetBeepDuration( DurationType type, Int16 beep_duration );
Int16 ScnGetBeepDuration( DurationType type );

//--------------------------------------------------------------------------------------------------

//Set sled function
Int16 ScnCmdTrigSledOn();
Int16 ScnCmdTrigSledOff();
Int16 ScnGetTrigSledMode();


#ifdef __cplusplus
    }
#endif

#endif