/******************************************************************************
 *
 * 	Barcode Scanner Common Defines.
 *
 *	Copyright (c) 2006	Janam Partners LLC. All rights reserved.
 *
 * 	File: 			ScanMgrStruct.h
 *
 * 	Release: 		Palm OS 5.x
 *
 * 	Description: 	Reserved for backward compatible
 *					Common structure for using the Scanner Library
 *
 * 	History:	
 *	Version 1.0		PT, JL			Oct, 2006
 *
 *****************************************************************************/
 
#ifndef __SCANMGRSTRUCT_H__
#define __SCANMGRSTRUCT_H__

//#pragma once on

#ifdef __cplusplus
   extern "C" {
#endif

#include "ScanMgrDef.h"

//extended defines and structure
#include "JanamExtendedDef.h"
#include "JanamExtendedStruct.h"

//ScanEvent Structure
typedef struct 
{
	eventsEnum     eType;
	Boolean        penDown;
	Int16          screenX;
	Int16          screenY;
	union scanData 
	{
		struct scanGen
		{
			UInt16 data1;
			UInt16 data2;		
			UInt16 data3;
			UInt16 data4;
			UInt16 data5;
			UInt16 data6;
			UInt16 data7;
			UInt16 data8;
		} scanGen;

		struct 
		{
			UInt16 batteryLevel;	
			UInt16 batteryErrorType;	
		} batteryError;

	} scanData;	// End of union
	
} ScanEventType;
typedef ScanEventType *ScanEventPtr;

//-----------------------------------------------------------------------------
//Maximum Packet Length
#define MAX_PACKET_LENGTH     						258

//Barcode Message Structure
typedef struct  tagMESSAGE
{
	Int16 length; 									//data length
	Int16 type;										//barcode type
	Int16 status; 									//decode status
	UInt8 data[MAX_PACKET_LENGTH]; 					//barcode data
} MESSAGE;

//-----------------------------------------------------------------------------
#define MAX_DCD_LENGTH       						258

//DCD Message Structure
typedef struct  tagMESSAGE_Dcd
{
	Int16 length; // length of the data
	Int16 type;	// contains the barcode type when the msg is DecodeData
	Int16 status; // should be STATUS_OK
	UInt8 symLetter;
	UInt8 symModifier;	
	UInt8 data[MAX_DCD_LENGTH]; // the message data
} MESSAGE_Dcd;

//-----------------------------------------------------------------------------
//Batch Structure
#define MAX_BATCH_PARAM         247
typedef struct tagBATCH
{
	Int16 length;
	Int16 data[MAX_BATCH_PARAM + 3];
} BATCH;

//-----------------------------------------------------------------------------
//Comm Structure. No longer in use
typedef struct  tagCOMM_STRUCT
{
	Int16 port;
	Int16 baud;
	Int16 parity;			
	Int16 stop_bits;
	Int16 host_timeout;
} COMM_STRUCT;

//-----------------------------------------------------------------------------
//Beep Sound enum
typedef enum tagBeepType
{
	No_Beep 			= 0x00,
	One_Short_High,
    Two_Short_High,
    Three_Short_High,
    Four_Short_High,
    Five_Short_High,

    One_Short_Low,
    Two_Short_Low,
    Three_Short_Low,
    Four_Short_Low,
    Five_Short_Low,

    One_Long_High,
    Two_Long_High,
    Three_Long_High,
    Four_Long_High,
    Five_Long_High,

    One_Long_Low,
    Two_Long_Low,
    Three_Long_Low,
    Four_Long_Low,
    Five_Long_Low,

    Fast_Warble,
    Slow_Warble,
    Mix1,
    Mix2,
    Mix3,
    Mix4,

   	Decode_Beep,
   	Bootup_Beep,
   	Parameter_Entry_Error_Beep,
   	Parameter_Defaults_Beep,
   	Parameter_Entered_Beep,
   	Host_Convert_Error_Beep,
   	Transmit_Error_Beep,
   	Parity_Error_Beep,
   	
   	Last_Beep
} BeepType;

//-----------------------------------------------------------------------------
//Conversion enum
typedef enum tagConvertType
{
	convertUpcEtoUpcA,
	convertUpcE1toUpcA,
	convertCode39toCode32,
	convertEan8toEan13,
	convertI2of5toEan13
} ConvertType; 

//-----------------------------------------------------------------------------
//Beep Duration enum
typedef enum
{
	decodeDuration 				= DECODE_BEEP_DUR,
	shortDuration 				= SHORT_BEEP_DUR,
	mediumDuration 				= MEDIUM_BEEP_DUR,
	longDuration 				= LONG_BEEP_DUR
} DurationType;

//Beep Frequency enum
typedef enum
{
	decodeFrequency 			= DECODE_BEEP_FREQ,
	lowFrequency 				= LOW_FREQ,
	mediumFrequency 			= MEDIUM_FREQ,
	highFrequency 				= HIGH_FREQ
} FrequencyType;

//-----------------------------------------------------------------------------
//Button Type enum
typedef enum
{
	buttonDateBook 				= 0,
	buttonAddressBook,
	buttonTodoList,
	buttonMemoPad
} ButtonType;

//-----------------------------------------------------------------------------
//Launch Code enum
typedef enum {
	myAppCmdBatteryAlert 		= sysAppLaunchCmdCustomBase,
	myAppCmdS24BatteryAlert, 
	myAppCmdNotUsed
} MyAppCustomActionCodes;

//-----------------------------------------------------------------------------
//Decode Parameters
#define PACKET_CONTINUATION		0x0002
#define MAX_DECODER_PARMS       3750


//-----------------------------------------------------------------------------
//Bar Code Enum Types with Extended Codes provided by Janam
typedef enum tagBarType
{
	barCODE39		= 0x00,
	barUPCA			= 0x01,
	barUPCE 		= 0x02,
	barEAN13		= 0x03,
	barEAN8			= 0x04,
	barD25			= 0x05,
	barI2OF5		= 0x06,
	barCODABAR		= 0x07,
	barCODE128		= 0x08,
	barCODE93		= 0x09,
	barTRIOPTIC39 	= 0x0D,
	barUCC_EAN128	= 0x0E,
	barMSI_PLESSEY  = 0x0B,
	barUPCE1		= 0x0C,
	barBOOKLAND_EAN = 0x53,
	barISBT128		= 0x54,
	barCOUPON		= 0x55,
	barCode32		= 0x56,
//EXTENDED Support	
	barAZTEC		= 0xC0,
	barCODE11		= 0xC1,
	barCODE49		= 0xC2,
	barCOMPOSITE	= 0xC3,
	barDATAMAXTRIX	= 0xC4,
	barMAXICODE		= 0xC5,
	barMICROPDF		= 0xC6,
	barOCR			= 0xC7,
	barPDF417		= 0xC8,	
	barPOSTNET		= 0xC9,
	barQR			= 0xCA,	
	barRSS			= 0xCB,	
	barBPO			= 0xCC,	
	barCANPOST		= 0xCD,
	barAUSPOST		= 0xCE,
	barIATA25		= 0xCF,
	barCODABLOCK	= 0xD0,
	barJAPOST		= 0xD1,
	barPLANET		= 0xD2,
	barDUTCHPOST	= 0xD3,
	barTLCODE39		= 0xD4,
	barMATRIX25		= 0xD5,
	barCHINAPOST	= 0xD6,
	barKOREAPOST	= 0xD7,
	barTELEPEN		= 0xD8,
	barCODE16K		= 0xD9,
	barPOSICODE		= 0xDA,
	barUSPS4CB		= 0xDB,
	barIDTAG		= 0xDC,
	barRSS_LIM		= 0xDD,
	barRSS_EXP		= 0xDE,
	barMSI			= 0xA0,
	barPlessey		= 0xA1,	
	barALL			= 0xFF
} BarType;

//-----------------------------------------------------------------------------
//Decode Option Structure
#define DEC_DECODE_OPTIONS_VER	0x03
typedef struct
{
	Int32			Version;
    Int32			PrintWeight;		// Valid range is 1-7
	Int32			DecodeMode;			// Full Omni, Reduced Omni, ALD
    Int32			LinearRange;		// Valid range 1-6
	Int32			VideoReverse;		// 0=disabled, 1=enabled
    Int32			DecAttemptLimit;    // 0 = disabled, Valid range 1-10,000
	Int32			SearchLimit;		// 0 = disabled, Valid range 1-10,000
	unsigned short	AddendaDelay;		// (search time for addenda in ms)
	unsigned short	UpcComDelay;		// (search time for 2D symbol in UPC COM mode)
	unsigned short	ConcatDelay;		// (search time for ISBT/Codabar concatenation)
	unsigned short	MldEnaMask;			// Per symbology MLD control
	Int32			Flags;				// special flags
} DecodeOptions_t;


#ifdef __cplusplus
    }
#endif


#endif